# (C)2018 Ingram Micro Inc. all Rights Reserve
import logging

from sh_base import CleanUpSHBase
from oacleanup.settings import OAModule, CleanUpSteps
from oacleanup.utils.packages import remove_packages_from_repo
from oacleanup.utils.billing import is_billing_installed
from poaupdater import uSysDB

logger = logging.getLogger()

help_link = 'https://kb.odin.com/en/132851'

indicator_services = [
    ('service', 'Apache'),
    ('service', 'Proftpd'),
]

common_lsh_services = [
    ('service', 'domain_parking'),
    ('service', 'shared_ssl_config'),
    ('other', 'apache-redirect-admin'),
    ('other', 'apache_common'),
    ('other', 'Apache-LimitConn'),
    ('other', 'apache-skeleton.default'),
    ('other', 'chrooted_ssh'),
]

services_to_remove = indicator_services + common_lsh_services + [
    ('service', 'WebFileManager'),
    ('service', 'crontabs'),
    ('service', 'logparser'),
    ('service', 'logrotate'),
    ('other', 'domain_parking_common'),
    ('other', 'apache-data'),
]

packages_to_remove_from_repo = indicator_services + common_lsh_services

LSH_MODULE_NAME = 'Linux Shared'


class CleanUpLSH(CleanUpSHBase):
    def __init__(self):
        list_of_available_steps = [
            CleanUpSteps.UNPOVISION_RESOURES,
            CleanUpSteps.DEACTIVATE_SERVICE_TEMPLATES,
            CleanUpSteps.CLEAR_SERVICE_NODES,
            CleanUpSteps.REMOVE_PACKAGES,
        ]
        if is_billing_installed():
            list_of_available_steps.insert(1, CleanUpSteps.SYNCHRONIZE_ONLINE_STORES)

        super(CleanUpLSH, self).__init__(
            name=OAModule.LSH,
            list_of_available_steps=list_of_available_steps,
            indicator_services=indicator_services,
            services_to_detach=indicator_services,
            module_services=services_to_remove,
            help_link=help_link,
        )

    def _remove_packages_from_repo(self, sc_list):
        return remove_packages_from_repo(packages_to_remove_from_repo, ['RHEL'])

    def _unlink_aps_application_from_subscriptions(self, subscription_ids):
        q = """
            UPDATE apache_vdirs SET is_in_use = 'n'
            WHERE homedir_id IN (
                SELECT hd_id FROM domain_services ds
                JOIN services s ON s.service_id = ds.service_id
                WHERE s.name IN %s AND ds.subscription_id IN %s
            )
        """

        con = uSysDB.connect()
        cur = con.cursor()
        cur.execute(q, (tuple([x for _, x in self.indicator_services]), tuple(subscription_ids)))
        con.commit()
