# (C)2018 Ingram Micro Inc. all Rights Reserve
import logging

from sh_base import CleanUpSHBase
from oacleanup.settings import OAModule, CleanUpSteps
from oacleanup.utils import packages, hosts, subscriptions, nodes
from poaupdater import uSysDB, openapi, apsapi
from oacleanup.utils.billing import is_billing_installed, get_billing_api

logger = logging.getLogger()

help_link = 'https://cloudblue.freshdesk.com/support/solutions/articles/44001882463'

WHL_SERVICES = [
    ('service', 'ApacheNG'),
    ('service', 'ProftpdNG')
]

WHL_MN_PACKAGES = [
    ('other', 'WebHostingLinuxApplication'),
    ('other', 'WebHosting-Linux-Endpoint'),
    ('other', 'WebHosting-Linux-APS-Package'),
    ('other', 'default_website_template'),
    ('other', 'PrivilegesWebLinux'),
    ('other', 'PrivilegesCron'),
    ('other', 'DatabaseRoles')
]

WHL_SN_PACKAGES = WHL_SERVICES + [
    ('service', 'WebCluster'),
    ('service', 'WebClusterExt'),
    ('service', 'WebFileManager'),
    ('service', 'DomainParkingNG')
]

NON_WHL_SN_PACKAGES = [
    ('service', 'crontabs'),
    ('service', 'logparser'),
    ('service', 'logrotate'),
    ('other', 'domain_parking_common')
]

WHL_PACKAGES_TO_REMOVE_FROM_REPO = WHL_MN_PACKAGES + WHL_SN_PACKAGES

WHL_SCS_EXCEPT = [
    'PgSQL',
    'WebFileManager',
    'MariaDB',
    'DomainParking',
    'ConfigurationManager',
    'DiskspaceQuotaManager',
    'InternalIPResolver',
    'CronManager',
]

WHL_SCS = ['Apache', 'ProFTPD', 'WebCluster'] + WHL_SCS_EXCEPT

WHL_MODULE_SHORT_NAME = 'Linux Shared'

class CleanUpWHL(CleanUpSHBase):
    def __init__(self):
        list_of_available_steps = [
            CleanUpSteps.UNPOVISION_RESOURES,
            CleanUpSteps.DEACTIVATE_SERVICE_TEMPLATES,
            CleanUpSteps.CLEAR_SERVICE_NODES,
            CleanUpSteps.UNINSTALL_MODULE,
            CleanUpSteps.DETACH_NODES,
        ]
        if is_billing_installed():
            list_of_available_steps.insert(1, CleanUpSteps.SYNCHRONIZE_ONLINE_STORES)

        super(CleanUpWHL, self).__init__(
            name=OAModule.WHL,
            list_of_available_steps=list_of_available_steps,
            indicator_services=WHL_SERVICES,
            services_to_detach=WHL_SERVICES,
            module_services=WHL_SN_PACKAGES + NON_WHL_SN_PACKAGES,
            help_link=help_link,
        )
        self.api = openapi.OpenAPI()
        if is_billing_installed():
            self.pba_api = get_billing_api()

    def _uninstall_module(self):
        scs_to_rm = WHL_SCS
        self._remove_resource_types(scs_to_rm)
        self._remove_resource_classes(scs_to_rm)
        self._drop_service_groups()
        self._drop_service_classes()
        self._unregister_domain_hostings()
        self._drop_scs_tables()
        self._drop_scs_views()
        self._remove_aps_application_resource_types('http://www.odin.com/webhosting-linux')
        self.rem_gdpr_and_app_instance()
        packages.remove_packages_on_hosts([1], WHL_MN_PACKAGES)
        self._remove_periodic_tasks(['taskCollectResourceUsage', 'initGetUsage', 'taskDBSCollectDiskSpaceUsagePeriodicHandler'])
        self.drop_sc('Apache')
        self.drop_sc('ProFTPD')
        self.drop_sc('WebCluster')
        self._remove_sc_packages_from_mn(WHL_SCS_EXCEPT)
        self._drop_scs_tables_seqs()

        self._remove_packages(scs_to_rm)
        if not self.is_module_exists("Linux Mail Hosting"):
            self.remove_common_scs()
        hosts.restart_pa_agent()
        self._unregister_module(WHL_MODULE_SHORT_NAME)
        hosts.restart_mn()
        hosts.restart_ui_hosts()

    def _drop_scs_tables(self):
        con = uSysDB.connect()
        cur = con.cursor()
        tables = ['apache_certificates',
                  'apache_denied_ips',
                  'apache_directives',
                  'apache_directories',
                  'apache_errcodes',
                  'apache_extensions',
                  'apache_handler_types',
                  'apache_hlp_allowed_urls',
                  'apache_hlp_extensions',
                  'apache_homedir_doc_bodies',
                  'apache_homedir_entries',
                  'apache_homedir_errdocs',
                  'apache_homedir_includes',
                  'apache_hostlist',
                  'apache_hotlink_protections',
                  'apache_include_config',
                  'apache_log_analyzer_services',
                  'apache_mime_types',
                  'apache_mime_types_extensions',
                  'apache_mirrors',
                  'apache_php_configurations',
                  'apache_resources',
                  'apache_service_templates',
                  'apache_services',
                  'apache_services_limits',
                  'apache_ssh_access',
                  'apache_sys_mime_types_extensions',
                  'apache_system_handlers_extensions',
                  'apache_system_mime_types',
                  'apache_user_handlers',
                  'apache_user_handlers_extensions',
                  'apache_userlist',
                  'apache_vdirs',
                  'apache_vdirs_php_settings',
                  'apache_vdirs_url_bindings',
                  'apache_vdirs_url_handlers',
                  'apache_vdirs_url_mappings',
                  'apache_vhost_limits',
                  'apache_virtual_dirs',
                  'apache_h2e_def_certificate',
                  'apache_h2e_clusters',
                  'apache_h2e_cluster_addon_services',
                  'apache_h2e_service_classes',
                  'apache_h2e_nodes',
                  'apache_h2e_mount_points',
                  'apache_h2e_ws_to_mpoint_affinity',
                  'apache_h2e_htaccess_directories',
                  'apache_h2e_ws_limits'
                  ]
        for table in tables:
            cur.execute("DROP TABLE IF EXISTS %s CASCADE" % table)
        con.commit()

    def _drop_scs_views(self):
        con = uSysDB.connect()
        cur = con.cursor()
        views = ['v_apache_h2e_mount_points',
                 'v_apache_h2e_load_balancers',
                 'v_apache_h2e_webservers',
                 'v_apache_h2e_clusters',
                 'v_apache_h2e_users_groups',
                 'v_apache_h2e_host_packages',
                 'v_apache_h2e_ssh_access']
        for view in views:
            cur.execute("DROP VIEW IF EXISTS %s CASCADE" % view)
        con.commit()

    def _drop_scs_tables_seqs(self):
        con = uSysDB.connect()
        cur = con.cursor()
        self._drop_seqs_deps(['apache_handler_types_seq',
                              'apache_include_config_seq',
                              'apache_system_mime_types_seq',
                              'apache_mime_types_seq',
                              'apache_user_handlers_seq',
                              'apache_directories_seq',
                              'apache_userlist_seq',
                              'apache_denied_ips_seq',
                              'apache_vdirs_seq',
                              'apache_extensions_seq',
                              'apache_h2e_def_certificate_seq',
                              'apache_h2e_nodes_seq',
                              'apache_h2e_mount_points_seq'
                              ],
                             cur)
        con.commit()

    def _drop_service_classes(self):
        con = uSysDB.connect()
        cur = con.cursor()
        cur.execute("DELETE FROM service_classes WHERE name = 'Apache' OR name = 'ProFTPD'")
        con.commit()

    def _drop_service_groups(self):
        con = uSysDB.connect()
        cur = con.cursor()
        if not uSysDB.table_exist(con, 'service_groups'):
            return
        cur.execute(
            "DELETE FROM service_groups WHERE group_name = 'Apache service default configuration' OR group_name = 'NG default configuration'")
        con.commit()

    def _remove_packages_from_repo(self, sc_list):
        count_packages = packages.remove_packages_from_repo(WHL_PACKAGES_TO_REMOVE_FROM_REPO)
        scs = [('sc', sc) for sc in sc_list]
        count_packages += packages.remove_packages_from_repo(scs, None)
        return count_packages

    def _unregister_domain_hostings(self):
        hostings = ['Apache WebSite', 'Domain Parking', 'Single Page', 'Frame Forwarding', 'Standart Forwarding']
        for h in hostings:
            self._unregister_domain_hosting(h)

    def _detach_nodes(self):
        logger.info('Detaching the %s nodes...' % self.name)
        hosts_with_service = self._get_host_ids_for_whl()

        subs = subscriptions.get_subscription_ids_by_hosts(hosts_with_service)
        backup_nodes = nodes.get_backup_nodes_by_subs(subs)
        if nodes.MN_NODE_ID in backup_nodes:
            logger.warning(
                'There are backups of subscriptions stored on the management node. '
                'If you want to keep them, move them away from the management node. '
                'Otherwise, they will be removed when subscriptions are destroyed.'
            )
            backup_nodes.remove(nodes.MN_NODE_ID)

        whl_related_nodes = hosts_with_service + backup_nodes
        if not whl_related_nodes:
            logger.info('There are no service nodes related to %s.', self.name)
            return

        logger.debug(
            'The following service nodes related to %s were found: %s.', self.name,
            ', '.join([str(host_id) for host_id in whl_related_nodes])
        )

        hosts_to_disable = hosts.filter_hosts_by_status(whl_related_nodes, False)
        if not hosts_to_disable:
            logger.info('All service nodes related to %s were already detached.', self.name)
        else:
            hosts.mark_hosts_physically_destroyed(hosts_to_disable, True)
            logger.info(
                'The following nodes were detached: %s. Now you can delete the subscriptions related to %s or you can '
                'switch them to another plan. After this is done, you need to attach the nodes back.',
                ", ".join([str(item) for item in hosts_to_disable]), self.name
            )

    def _get_host_ids_for_whl(self):
        q = 'SELECT ahh.host_id FROM apache_h2e_nodes ahh WHERE ahh.host_id IS NOT NULL';
        con = uSysDB.connect()
        cur = con.cursor()
        cur.execute(q)
        return [x for x, in cur.fetchall()]

    def _unregister_domain_hosting(self, hosting_name):
        con = uSysDB.connect()
        cur = con.cursor()
        cur.execute("DELETE FROM domain_hosting_configurators WHERE dhtype = '%s'" % hosting_name)
        cur.execute("DELETE FROM domain_hosting_type_categories WHERE dhtype = '%s'" % hosting_name)
        cur.execute("DELETE FROM domain_hosting_types WHERE dhtype = '%s'" % hosting_name)
        con.commit()

    def _unlink_aps_application_from_subscriptions(self, subscription_ids):
        q = """
            UPDATE apache_vdirs SET is_in_use = 'n'
            WHERE homedir_id IN (
                SELECT hd_id FROM domain_services ds
                JOIN services s ON s.service_id = ds.service_id
                WHERE s.name IN %s AND ds.subscription_id IN %s
            )
        """

        con = uSysDB.connect()
        cur = con.cursor()
        cur.execute(q, (tuple([x for _, x in WHL_SERVICES]), tuple(subscription_ids)))
        con.commit()

    def rem_gdpr_and_app_instance(self):
        # Application instance should be removed to avoid using remove_app_instance method of config_utils.py.
        # Mentioned method starts during package removing, the method contains error and does not work.
        # Application instance does not allow deletion if gdpr-management resource still exists. So it should be removed also.
        con = uSysDB.connect()
        cur = con.cursor()
        cur.execute("select * from aps_applications where app_uid = 'http://www.odin.com/webhosting-linux'")
        result = cur.fetchone()
        if not result:
            return
        app_id = result[0]

        result = self.api.pem.APS.getApplicationInstances(app_id=app_id)
        if not result:
            return
        app_instance_id = result[0]['application_instance_id']

        cur.execute("select uuid from aps_application_instances where app_instance_id = %s" % app_instance_id)
        app_instance_uuid = cur.fetchone()[0]

        ret = self.api.pem.APS.getApplicationInstanceToken(
            application_instance_id=app_instance_id
        )
        token = ret['aps_token']
        aapi = apsapi.API(url='https://localhost:6308')

        resp = aapi.GET(
            path='/aps/2/resources?implementing(http://www.odin.com/whl-gdpr-support/)',
            headers={"APS-Token": token},
        )
        for res in resp:
            path = '/aps/2/applications/%s/whl-gdpr-management/%s' % (app_instance_uuid, res.aps.id)
            aapi.DELETE(
                path=path,
                headers={"APS-Token": token}
            )

        request_id = self.api.beginRequest()
        self.api.pem.APS.unprovideApplicationInstance(application_instance_id=app_instance_id)
        self.api.commit()
        openapi.waitRequestComplete(request_id, "unprovideApplicationInstance", True)
