# (C)2018 Ingram Micro Inc. all Rights Reserved
from poaupdater import uLogging

import argparse
import logging
import sys

uLogging.log_to_console = False

from settings import OAModule, LOG_FILE_NAME, OA_NAME, BillingModule

uLogging.logfile = file(LOG_FILE_NAME, 'a')

from cleanup.lsh import CleanUpLSH
from cleanup.whl import CleanUpWHL
from cleanup.wsh import CleanUpWSH
from cleanup.hosted_exchange import CleanUpHostedExchange
from cleanup.cpp_domain_registrars import CleanUpCppDomainRegistrars
from cleanup.cpp_cert_gate import CleanUpCppCertGate
from cleanup.qmail import CleanUpQmail
from cleanup.lync import CleanUpLync
from cleanup.whi import CleanUpWHI
from oacleanup.utils.hosts import is_pa_agent_started
from utils.errors import CleanUpException

logger = logging.getLogger()


def configure_logging():
    logger.setLevel(logging.DEBUG)
    formatter = logging.Formatter('[%(asctime)s] [%(levelname)s]: %(message)s')
    stdout_logger_handler = logging.StreamHandler(sys.stdout)
    stdout_logger_handler.setLevel(logging.INFO)
    stdout_logger_handler.setFormatter(formatter)
    logger.addHandler(stdout_logger_handler)

    formatter = logging.Formatter('[%(asctime)s] [%(levelname)s]: %(message)s')
    file_logger_handler = logging.FileHandler(LOG_FILE_NAME)
    file_logger_handler.setLevel(logging.NOTSET)
    file_logger_handler.setFormatter(formatter)
    logger.addHandler(file_logger_handler)


def run(clean_up_module_name):
    configure_logging()
    try:
        if not is_pa_agent_started():
            raise CleanUpException(
                'The pa-agent service is not running on the management node. Start this service to continue with the '
                'cleanup process.'
            )

        if clean_up_module_name == OAModule.LSH:
            clean_up_module = CleanUpLSH()
        elif clean_up_module_name == OAModule.WHL:
            clean_up_module = CleanUpWHL()
        elif clean_up_module_name == OAModule.WSH:
            clean_up_module = CleanUpWSH()
        elif clean_up_module_name == OAModule.EXCHANGE:
            clean_up_module = CleanUpHostedExchange()
        elif clean_up_module_name == OAModule.QMAIL:
            clean_up_module = CleanUpQmail()
        elif clean_up_module_name == OAModule.LYNC:
            clean_up_module = CleanUpLync()
        elif clean_up_module_name == OAModule.WHI:
            clean_up_module = CleanUpWHI()
        elif clean_up_module_name == BillingModule.CPP_DOMAIN_REGISTRARS:
            clean_up_module = CleanUpCppDomainRegistrars()
        elif clean_up_module_name == BillingModule.CPP_CERT_GATE:
            clean_up_module = CleanUpCppCertGate()
        else:
            raise NotImplementedError(
                'The procedure of cleaning up the module %s is not available yet.' % clean_up_module_name
            )

        args = parse_options(clean_up_module)
        logger.info(
            'The cleanup tool for %s was started: %s %s.',
            clean_up_module.name, sys.argv[0], ' '.join("'%s'" % arg for arg in sys.argv[1:])
        )
        result = clean_up_module.run(**args)
        logger.info("Operation has been successfully finished. See more details in the log file '%s'.", LOG_FILE_NAME)
        return result
    except SystemExit as e:
        return e.code
    except KeyboardInterrupt:
        logger.warning('The process was interrupted.')
        return 1
    except Exception, e:
        logger.exception('An unexpected error occurred: %s', str(e))
        logger.error("An unexpected error occurred. See details in the log file '%s'.", LOG_FILE_NAME)
        return 1


def parse_options(clean_up_module):
    list_of_available_steps = clean_up_module.list_of_available_steps
    parser = argparse.ArgumentParser(
        description='Use this tool to completely remove the %s module from your installation of %s. '
                    'For more information, see %s'
                    % (clean_up_module.name, OA_NAME, clean_up_module.help_link),
        formatter_class=lambda prog: argparse.HelpFormatter(
            prog=prog,
            width=110,
        ),
    )
    parser.add_argument('--check-only', action='store_true', help='Perform checks to detect possible issues.')
    subparser = parser.add_subparsers(dest='clean_up_step', metavar='Command')
    for cmd in list_of_available_steps:
        created_parser = subparser.add_parser(cmd.name, help=cmd.description({'module': clean_up_module.name}))
        for arg_name, arg_desc in cmd.arguments.iteritems():
            created_parser.add_argument(arg_name, action='store', help=arg_desc, required=True)

    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(1)

    return vars(parser.parse_args())
