# (C)2018 Ingram Micro Inc. all Rights Reserved
from collections import namedtuple
import os
import __main__

Step = namedtuple('Step', ('name', 'description', 'function', 'checks', 'arguments'))

LOG_FILE_NAME = '%s.log' % os.path.splitext(__main__.__file__)[0]
OA_NAME = 'CloudBlue Commerce'


def formatter(description):
    def formatter_curried(params):
        return description.format(**params)

    return formatter_curried


class OAModule(object):
    WSH = 'Windows Shared Hosting'
    LSH = 'Linux Shared Hosting'
    WHL = 'WebHosting Linux'
    EXCHANGE = 'Hosted Exchange'
    QMAIL = 'Qmail'
    LYNC = 'Lync'
    WHI = 'Windows Hosting Infrastructure'


class BillingModule(object):
    CPP_DOMAIN_REGISTRARS = 'C++ Domain Registrars'
    CPP_CERT_GATE = 'C++ SSL Certificates in Billing'


class CleanUpSteps(object):
    DETACH_NODES = Step(
        name='detach-nodes',
        description=formatter(
            'Detach nodes with {module}. After this, %s can not operate on that nodes.' % OA_NAME
        ),
        function='_detach_nodes',
        checks=[],
        arguments={},
    )
    ATTACH_NODES = Step(
        name='attach-nodes',
        description=formatter(
            'Attach nodes with {module} in order to %s can continue operating on these nodes.' % OA_NAME
        ),
        function='_attach_nodes',
        checks=['_check_nodes_list'],
        arguments={'--node-ids': 'Node IDs that should be attach to %s.' % OA_NAME},
    )
    UNPOVISION_RESOURES = Step(
        name='unprovision-resources',
        description=formatter('Unprovision resources related to {module} from subscriptions.'),
        function='_unprovision_resources',
        checks=[],
        arguments={'--subscription-ids': 'The IDs of subscriptions from which resources related to {module} must be unprovisioned.'},
    )
    DEACTIVATE_SERVICE_TEMPLATES = Step(
        name='deactivate-service-templates',
        description=formatter('Deactivate all service templates related to the {module} module.'),
        function='_deactivate_service_templates',
        checks=['_show_service_templates'],
        arguments={},
    )
    SYNCHRONIZE_ONLINE_STORES = Step(
        name='synchronize-online-stores',
        description=formatter('Synchronize all online stores to unpublish the service plans '
                              'that are based on the deactivated service templates. '
                              'Note that during the synchronization, all other settings, '
                              'even those that are not related to service plans, will be synchronized as well.'),
        function='_synchronize_online_stores',
        checks=['_is_service_templates_deactivated'],
        arguments={},
    )
    CLEAR_SERVICE_NODES = Step(
        name='remove-packages-from-service-nodes',
        description=formatter('Remove all packages related to the {module} module from service nodes.'),
        function='_remove_packages_from_service_nodes',
        checks=['_is_service_templates_deactivated', '_is_subscriptions_destroyed'],
        arguments={},
    )
    UNINSTALL_MODULE = Step(
        name='uninstall-module',
        description=formatter(
            'Remove all packages from the management node and repository, uninstall modules and roles related to the '
            '{module} module.'
        ),
        function='_uninstall_module',
        checks=['_is_service_templates_deactivated', '_is_subscriptions_destroyed', '_is_service_nodes_cleared'],
        arguments={},
    )
    REMOVE_PACKAGES = Step(
        name='remove-packages-from-repository',
        description=formatter('Remove all packages related to the {module} from the repository.'),
        function='_remove_packages',
        checks=['_is_service_templates_deactivated', '_is_subscriptions_destroyed', '_is_service_nodes_cleared'],
        arguments={},
    )
