# (C)2018 Ingram Micro Inc. all Rights Reserve
from poaupdater import uPackaging, uSysDB, uUtil, uPEM, uHCL

import logging

from oacleanup.utils.errors import CleanUpException

logger = logging.getLogger()


def get_host_ids_by_services(services):
    package_hosts = [uPackaging.getPackageHosts(service).values() for service in services]
    package_hosts = [item for sublist in package_hosts for subsublist in sublist for item in subsublist]

    return list(set(package_hosts))


def mark_hosts_physically_destroyed(host_ids, is_physically_destroyed):
    if not host_ids:
        logger.debug('There are no hosts to mark as detached.')
        return

    q = 'UPDATE hosts SET is_physically_destroyed=%s WHERE host_id IN %s'
    con = uSysDB.connect()
    cur = con.cursor()
    cur.execute(q, 'y' if is_physically_destroyed else 'n', tuple(host_ids))
    con.commit()

    if is_physically_destroyed:
        logger.debug('The hosts with these IDs are marked as decommissioned: %s.' %
                     ', '.join([str(host_id) for host_id in host_ids]))
    else:
        logger.debug('The hosts with these IDs are operating properly: %s.' %
                     ', '.join([str(host_id) for host_id in host_ids]))


def filter_hosts_by_status(host_ids, is_physically_destroyed):
    q = 'SELECT host_id FROM hosts WHERE is_physically_destroyed=%s AND host_id IN %s'
    con = uSysDB.connect()
    cur = con.cursor()
    cur.execute(q, 'y' if is_physically_destroyed else 'n', tuple(host_ids))
    return [x for x, in cur.fetchall()]


def is_pa_agent_started():
    try:
        uUtil.execCommand('service pa-agent status', [0])
        return True
    except Exception:
        return False


def restart_pa_agent():
    logger.info("Restarting the pa-agent service...")
    try:
        logger.debug("Stopping the pa-agent service...")
        uUtil.execCommand('service pa-agent stop', [0, 1, 123])
        logger.debug("The pa-agent service is stopped.")
    except Exception:
        raise CleanUpException('The pa-agent service cannot be stopped. You must restart it manually.')

    try:
        logger.debug("Starting the pa-agent service...")
        uUtil.execCommand('service pa-agent start')
        logger.debug("The pa-agent service is started.")
    except Exception:
        raise CleanUpException('The pa-agent service cannot be started. You must start it manually.')

    logger.info("The pa-agent service is restarted.")


def restart_mn():
    logger.info("Restarting MN...")
    uPEM.stopMN()
    uPEM.startMN()
    logger.info("MN is restarted.")


def restart_ui_hosts():
    logger.info("Restarting UI hosts...")
    ui_hosts = [host_id for host_id in uPEM.getAllUiHosts() if host_id != 1]

    for h in ui_hosts:
        logger.info("Restart UI on the host #%s." % h)
        request = uHCL.Request(h, user='root', group='root')
        request.command('service pau stop && service pau start', stdout='stdout', stderr='stderr')
        request.perform()
    logger.info("UI hosts are restarted.")


def remove_files(host_id=None, files=None, force=False):
    if files is None or host_id is None:
        return
    request = uHCL.Request(host_id, user='root', group='root')
    for f in files:
        request.command('rm %s%s' % ('-f ' if force else '', f), stdout='stdout', stderr='stderr')
        try:
            request.perform()
        except uUtil.ExecFailed:
            pass   # The file has been already removed


def file_exists(host_id, file_path):
    if host_id is None or file_path is None:
        raise ValueError('Unable to check if a file exists: host ID and file path cannot be empty')

    request = uHCL.Request(host_id, user='root', group='root')
    request.command('ls --all %s' % file_path, stdout='stdout', stderr='stderr')
    try:
        request.perform()
        return True
    except uUtil.ExecFailed:
        return False
