# (C)2018 Ingram Micro Inc. all Rights Reserve
from poaupdater import uSysDB

from oacleanup.utils.errors import CleanUpException

RESELLERS_INDICATOR_RESOURCE_NAME = 'clients'


def get_subscription_ids_by_hosts(host_ids):
    if not host_ids:
        raise CleanUpException('Host identifiers were not set.')

    q = ('SELECT DISTINCT sub.sub_id FROM subscriptions sub'
         ' JOIN domain_services ds ON ds.subscription_id = sub.sub_id'
         ' JOIN services s ON s.service_id = ds.service_id'
         ' WHERE s.host_id IN %s')
    con = uSysDB.connect()
    cur = con.cursor()
    cur.execute(q, (tuple(host_ids),))

    return [subscription_id for [subscription_id] in cur.fetchall()]


def get_resellers_subscription_ids():
    q = ('SELECT ss.sub_id FROM subscriptions ss '
         'JOIN subs_resources sr ON ss.sub_id=sr.sub_id '
         'JOIN resource_types rt ON sr.rt_id=rt.rt_id '
         'JOIN resource_classes rc ON rt.class_id=rc.class_id '
         'WHERE rc.name = %s AND ss.sub_id != %s')
    con = uSysDB.connect()
    cur = con.cursor()
    cur.execute(q, (RESELLERS_INDICATOR_RESOURCE_NAME, 1))

    return [subscription_id for [subscription_id] in cur.fetchall()]


def get_customers_subscription_ids_by_hosts(host_ids):
    hosts_subscription_ids = get_subscription_ids_by_hosts(host_ids)
    reseller_subscription_ids = get_resellers_subscription_ids()

    return list(set(hosts_subscription_ids).difference(reseller_subscription_ids))


def get_resellers_subscription_ids_by_hosts(host_ids):
    hosts_subscription_ids = get_subscription_ids_by_hosts(host_ids)
    reseller_subscription_ids = get_resellers_subscription_ids()

    return list(set(hosts_subscription_ids).intersection(reseller_subscription_ids))


def get_subscriptions_with_databases(subscription_ids):
    con = uSysDB.connect()
    cur = con.cursor()
    sql = "SELECT sub_id FROM dbm_provide_info WHERE dbm_provide_info.sub_id IN %s"
    cur.execute(sql, (tuple(subscription_ids),))
    return [x for (x, ) in cur.fetchall()]


def unlink_databases_from_subscriptions(subscription_ids):
    con = uSysDB.connect()
    cur = con.cursor()

    cur.execute(
        """
        DELETE
        FROM dbm_users
        USING dbm_db_users, dbm_databases, dbm_provide_info
        WHERE
        dbm_users.user_id = dbm_db_users.user_id
        AND dbm_db_users.db_id = dbm_databases.db_id
        AND dbm_provide_info.provide_id = dbm_databases.provide_id
        AND dbm_provide_info.sub_id IN %s
        """, (tuple(subscription_ids),)
    )

    cur.execute(
        """
        DELETE
        FROM dbm_db_users
        USING dbm_databases, dbm_provide_info
        WHERE
        dbm_db_users.db_id = dbm_databases.db_id
        AND dbm_provide_info.provide_id = dbm_databases.provide_id
        AND dbm_provide_info.sub_id IN %s
        """, (tuple(subscription_ids),)
    )

    cur.execute(
        """
        DELETE
        FROM dbm_db_access_hosts
        USING dbm_databases, dbm_provide_info
        WHERE
        dbm_db_access_hosts.db_id = dbm_databases.db_id
        AND dbm_provide_info.provide_id = dbm_databases.provide_id
        AND dbm_provide_info.sub_id IN %s
        """, (tuple(subscription_ids),)
    )

    cur.execute(
        """
        DELETE
        FROM dbm_db_diskspace_elems
        USING dbm_databases, dbm_provide_info
        WHERE
        dbm_db_diskspace_elems.db_id = dbm_databases.db_id
        AND dbm_provide_info.provide_id = dbm_databases.provide_id
        AND dbm_provide_info.sub_id IN %s
        """, (tuple(subscription_ids),)
    )

    cur.execute(
        """
        DELETE
        FROM dbm_databases
        USING dbm_provide_info
        WHERE
        dbm_provide_info.provide_id = dbm_databases.provide_id
        AND dbm_provide_info.sub_id IN %s
        """, (tuple(subscription_ids),)
    )

    cur.execute(
        """
        DELETE
        FROM dbm_provide_info
        WHERE
        dbm_provide_info.sub_id IN %s
        """, (tuple(subscription_ids),)
    )
    con.commit()
