import logging
import time

logger = logging.getLogger()


class Task:
    def __init__(self, task_id, method, name, status):
        self.task_id = task_id
        self.method = method
        self.name = name
        self.status = status

    def __str__(self):
        return 'Task{task_id: %s, method: %s, name: "%s", status: %s}' % (self.task_id, self.method, self.name, self.status)

    def __repr__(self):
        return self.__str__()

    def delete(self, dbcur):
        self._wait_while_running(dbcur)
        self._delete(dbcur)

    def _delete(self, dbcur):
        logger.debug("Delete task %s" % self)
        dbcur.execute('DELETE FROM tm_params WHERE task_id = %s' % self.task_id)
        dbcur.execute('DELETE FROM tm_weights WHERE task_id = %s' % self.task_id)
        dbcur.execute('DELETE FROM tm_task_references WHERE task_id = %s' % self.task_id)
        dbcur.execute('DELETE FROM tm_blocker_tasks WHERE task_id = %s' % self.task_id)
        dbcur.execute('DELETE FROM tm_blocker_mutexes WHERE task_id = %s' % self.task_id)
        dbcur.execute('DELETE FROM tm_usual WHERE task_id = %s' % self.task_id)
        dbcur.execute('DELETE FROM tm_periodic WHERE task_id = %s' % self.task_id)
        dbcur.execute('DELETE FROM tm_tasks WHERE task_id = %s' % self.task_id)

    def _wait_while_running(self, dbcur):
        logger.debug('Wait while task %s is executing' % self.task_id)
        while self._is_running(dbcur):
            time.sleep(1)
        logger.debug(self)

    def _is_running(self, dbcur):
        dbcur.execute('SELECT status FROM tm_tasks WHERE task_id = %s' % self.task_id)
        result = dbcur.fetchone()
        if result:
            return result[0] == 'e'
        return False


def find_tasks(dbcur, method_name):
    logger.debug('Search tasks by method %s' % method_name)
    dbcur.execute("SELECT task_id, method, name, status FROM tm_tasks WHERE method = '%s'" % method_name)
    tasks = [Task(r[0], r[1], r[2], r[3]) for r in dbcur.fetchall()]
    logger.debug('Found tasks: %s' % tasks)
    return tasks
